#ifndef _IMAGE_H
#define _IMAGE_H

// Copyright (c) OpenMedia Lab., 2013. ver.8/14/2013

#include <stdio.h>
#include "color.h"

class Image {
public:
	unsigned char *pv;   // pixel values RGB RGB RGB...
	int      width;
	int      height;
	int      channel; // grey:1 | RGB:3 | RGBA:4
	int      dpiH;
	int      dpiV;
	void set( int i, int j, int c, unsigned char d ){ pv[(i+j*width)*channel+c] = d; }
	void set( int i, int j, unsigned char d ){ pv[i+j*width] = d; }
	void set( unsigned char d, int i, int j, int c ){ this->set( i, j, c, d ); }
	void set( unsigned char d, int i, int j ){ this->set( i, j, d ); }
	void set( int i, int j, int d  ){ this->set( i, j, cramp(d) ); }
	void setRed( int i, int j, unsigned char d ){ pv[(i+j*width)*channel] = d; }
	void setRed( unsigned char d, int i, int j ){ this->setRed( i, j, d ); }
	void setRed( int i, int j, int d ){ this->setRed( i, j, cramp(d) ); }
	void setGreen( int i, int j, unsigned char d ){ pv[(i+j*width)*channel+1] = d; }
	void setGreen( unsigned char d, int i, int j ){ this->setGreen( i, j, d ); }
	void setGreen( int i, int j, int d ){ this->setGreen( i, j, cramp(d) ); }
	void setBlue( int i, int j, unsigned char d ){ pv[(i+j*width)*channel+2] = d; }
	void setBlue( unsigned char d, int i, int j ){ this->setBlue( i, j, d ); }
	void setBlue( int i, int j, int d ){ this->setBlue( i, j, cramp(d) ); }
	void set( int i, int j, unsigned char r, unsigned char g, unsigned char b ){ unsigned char *p = &pv[(i+j*width)*3]; *p++ = r; *p++ = g; *p = b; }
	void set( int i, int j, int r, int g, int b ){ this->set( i, j, cramp(r), cramp(g), cramp(b) ); }
	void set( int i, int j, unsigned char r, unsigned char g, unsigned char b, unsigned char a ){ unsigned char *p = &pv[(i+j*width)*4]; *p++ = r; *p++ = g; *p++ = b; *p = a; }
	unsigned char density( int i, int j, int c = 0 ){ return pv[(i+j*width)*channel+c]; }
	unsigned char red( int i, int j ){ return pv[(i+j*width)*channel]; }
	unsigned char green( int i, int j ){ return pv[(i+j*width)*channel+1]; }
	unsigned char blue( int i, int j ){ return pv[(i+j*width)*channel+2]; }
	void print( int i, int j, int c = 0 ){ printf("%d\n", pv[(i+j*width)*channel+c]); }
	void construct(){ if( pv != NULL ) delete pv; pv = new unsigned char[width*height*channel]; }
	void construct( int w, int h, int c = 1 ){ width=w; height=h; channel=c; this->construct(); }
	void copy( Image &image ){ if( image.width != width || image.height != height || image.channel != channel ) this->construct( image.width, image.height, image.channel ); for( long i=0; i<width*height*channel; i++ ) pv[i] = image.pv[i]; }
	void smooth( Image &original, int maskX, int maskY ){
		this->construct( original.width, original.height, original.channel );
		for( int c=0; c<channel; c++ ){
		  for( int j=0; j<height; j++ ){
			for( int i=0; i<width; i++ ){
				int density = 0;
				int n = 0;
				for( int i1=-maskX/2; i1<=maskX/2; i1++ ){
				for( int j1=-maskY/2; j1<=maskY/2; j1++ ){
					if( i+i1 >=0 && i+i1 <width && j+j1 >=0 && j+j1 <height ){
						density += original.density( i+i1, j+j1, c );
						n++;
					}
				}}
				density /= n;
				this->set( i, j, c, (unsigned char)(density) );
			}
		  }
		}
	}
	int load( char *filename ){
		FILE *fp;
		if( fopen_s( &fp, filename, "rb" ) !=0 ) return 1;
		fread(pv,width*height*channel,1,fp);
		fclose( fp );
		return 0;
	}
	int save( char *filename ){
		FILE *fp;
		if( fopen_s( &fp, filename, "wb" ) !=0 ) return 1;
		fwrite(pv,width*height*channel,1,fp);
		fclose( fp );
		return 0;
	}
#ifdef __GL_H__
	unsigned long format(){ if(channel==1) return GL_LUMINANCE; else if(channel==3) return GL_RGB; else if(channel==4) return GL_RGBA; else return 0; }
	void draw(int x, int y){
		glPixelZoom( 1.0, -1.0 ); 
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glRasterPos2i(-1,1);
		glDrawPixels( width, height, this->format(), GL_UNSIGNED_BYTE, pv );
	}
	void draw(){ this->draw(0,0); }
	void scale4Texture(){
		int w = 1; while( w <= width ) w *= 2; w /= 2;
		int h = 1; while( h <= height ) h *= 2; h /= 2;
		if( w != width || h != height ){
			Image image( w, h, channel );
			gluScaleImage( this->format(), width, height, GL_UNSIGNED_BYTE, pv, image.width, image.height, GL_UNSIGNED_BYTE, image.pv );
			this->copy( image );
		}
	}
	void readPixels(){
		GLint p[4];
		glGetIntegerv( GL_VIEWPORT, p );
		this->construct( p[2], p[3], 3 );
		glReadPixels( p[0],p[1],p[2],p[3], GL_RGB, GL_UNSIGNED_BYTE, pv );
	}
	int  saveScreen( char *filename ){ this->readPixels(); return this->save( filename ); }
#endif
	Image( int w, int h, int c = 1 ){  pv=NULL; this->construct( w, h, c ); dpiH = dpiV = 300; }
	Image( char *filename, int w, int h, int c = 1 ){  pv=NULL; this->construct( w, h, c ); dpiH = dpiV = 300; if( load( filename ) ) printf( "%s not found!\n", filename ); }
	Image( Image &image ){  pv=NULL; this->construct( image.width, image.height, image.channel ); }
	Image(void){ pv=NULL; }
	~Image(){ if( pv!= NULL ) delete pv; }
};

#endif //_IMAGE_H
