#ifndef _MATRIX_H
#define _MATRIX_H

// Copyright (c) OpenMedia Lab., 2013. ver.8/14/2013

#include <stdio.h>
#include <math.h>
#include "vector.h"

int inverseMatrix(double *im,double *m,int size){
int	i, j, k,j2;
int *num, *pivot ;
double pv, *a, *ia, atmp, *p1, *p2, *p3;
	num = new int[size];
	pivot = new int[size];
	a  = new double[size*size];
	ia = new double[size*size];
	
	for(k = 0, p1=m, p2=a, p3=ia; k < size*size; k++){
		*p2++ = *p1++;
		*p3++ = 0.0;
	}

	for(k = 0; k < size; k++){
		ia[k*size+k] = 1.0;
		num[k] = 1;
	}
	for(k = 0; k < size; k++) { /* forward */
		pv = 0.0;
		for(i = 0; i < size; i++) { /* search pivot */
			if( num[i] ){
				if(fabs(a[i*size+k]) > fabs(pv)){
					pv = a[i*size+k];
					pivot[k] = i;
				}
			 }
		}
		if( fabs(pv) <= DBL_EPSILON ) return 1;
		for(j = k; j < size; j++)  a[pivot[k]*size+j] /= pv;
		for(j = 0; j < size; j++) ia[pivot[k]*size+j] /= pv;
		num[pivot[k]] = 0;
		for(i = 0; i < size ; i++) { /* do other lines */
		    if(num[i]){
			atmp = a[i*size+k];
			for(j = k+1; j < size; j++)
			    a[i*size+j] -=  a[pivot[k]*size+j] * atmp;
			for(j = 0; j < size; j++)
			    ia[i*size+j] -= ia[pivot[k]*size+j] * atmp;
		    }
		}
	}
	for(k = size-2; k >= 0; k--){ /* back */
	    for(j = k+1; j < size; j++){
		atmp = a[pivot[k]*size+j];
		    for(j2 = 0; j2 < size; j2++)
			ia[pivot[k]*size+j2] -= ia[pivot[j]*size+j2] * atmp;
	    }
	}
	for(i = 0; i < size; i++){
		for(j = 0; j < size; j++){
			im[i*size+j]=ia[pivot[i]*size+j];
		}
	}
	delete num;
	delete pivot;
	delete a;
	delete ia;
	return 0;
}

class Matrix4X4{
public:
	double e[16];

	void add( Matrix4X4 &a ){ for( int i=0; i<16; i++ ) e[i] += a.e[i]; }
	void add( Matrix4X4 &a, Matrix4X4 &b ){ for( int i=0; i<16; i++ ) e[i] = a.e[i] + b.e[i]; }
	void set( double a1, double a2, double a3, double a4, double b1, double b2, double b3, double b4,
		double c1, double c2, double c3, double c4, double d1, double d2, double d3, double d4 ){
		e[0]=a1; e[1]=a2; e[2]=a3; e[3]=a4; e[4]=b1; e[5]=b2; e[6]=b3; e[7]=b4;
		e[8]=c1; e[9]=c2; e[10]=c3; e[11]=c4; e[12]=d1; e[13]=d2; e[14]=d3; e[15]=d4; }
	void set( double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3 ){
		e[0]=a1; e[1]=a2; e[2]=a3; e[4]=b1; e[5]=b2; e[6]=b3; e[8]=c1; e[9]=c2; e[10]=c3; }
	void setColumn( int col, double a, double b, double c, double d ){ e[col]=a; e[col+4]=b; e[col+8]=c; e[col+12]=d; }
	void setRow( int row, double a, double b, double c, double d ){ e[row*4]=a; e[row*4+1]=b; e[row*4+2]=c; e[row*4+3]=d; }
	void exchange( Matrix4X4 &a ){ Matrix4X4 b = a; a = *this; *this = b; }
	int  inverse( Matrix4X4 &m ){ return inverseMatrix( e, m.e, 4 ); }
	int  inverse(){ Matrix4X4 m = *this; return inverseMatrix( e, m.e, 4 ); }
	void multiply( double k ){ for( int i=0; i<16; i++) e[i]*=k; }
	void multiply( double k, Matrix4X4 &m ){ for( int i=0; i<16; i++) e[i]=k*m.e[i]; }
	void multiply( Matrix4X4 &m, double k ){ this->multiply( k, m ); }
	void multiply( Matrix4X4 &a, Matrix4X4 &b ){
		e[0] = a.e[0]*b.e[0] + a.e[1]*b.e[4] + a.e[2]*b.e[8] + a.e[3]*b.e[12];
		e[1] = a.e[0]*b.e[1] + a.e[1]*b.e[5] + a.e[2]*b.e[9] + a.e[3]*b.e[13];
		e[2] = a.e[0]*b.e[2] + a.e[1]*b.e[6] + a.e[2]*b.e[10] + a.e[3]*b.e[14];
		e[3] = a.e[0]*b.e[3] + a.e[1]*b.e[7] + a.e[2]*b.e[11] + a.e[3]*b.e[15];
		e[4] = a.e[4]*b.e[0] + a.e[5]*b.e[4] + a.e[6]*b.e[8] + a.e[7]*b.e[12];
		e[5] = a.e[4]*b.e[1] + a.e[5]*b.e[5] + a.e[6]*b.e[9] + a.e[7]*b.e[13];
		e[6] = a.e[4]*b.e[2] + a.e[5]*b.e[6] + a.e[6]*b.e[10] + a.e[7]*b.e[14];
		e[7] = a.e[4]*b.e[3] + a.e[5]*b.e[7] + a.e[6]*b.e[11] + a.e[7]*b.e[15];
		e[8] = a.e[8]*b.e[0] + a.e[9]*b.e[4] + a.e[10]*b.e[8] + a.e[11]*b.e[12];
		e[9] = a.e[8]*b.e[1] + a.e[9]*b.e[5] + a.e[10]*b.e[9] + a.e[11]*b.e[13];
		e[10] = a.e[8]*b.e[2] + a.e[9]*b.e[6] + a.e[10]*b.e[10] + a.e[11]*b.e[14];
		e[11] = a.e[8]*b.e[3] + a.e[9]*b.e[7] + a.e[10]*b.e[11] + a.e[11]*b.e[15];
		e[12] = a.e[12]*b.e[0] + a.e[13]*b.e[4] + a.e[14]*b.e[8] + a.e[15]*b.e[12];
		e[13] = a.e[12]*b.e[1] + a.e[13]*b.e[5] + a.e[14]*b.e[9] + a.e[15]*b.e[13];
		e[14] = a.e[12]*b.e[2] + a.e[13]*b.e[6] + a.e[14]*b.e[10] + a.e[15]*b.e[14];
		e[15] = a.e[12]*b.e[3] + a.e[13]*b.e[7] + a.e[14]*b.e[11] + a.e[15]*b.e[15];
	}
	void print(){ for( int i=0; i<4; i++ ) printf( "%f %f %f %f\n", e[i*4], e[i*4+1], e[i*4+2], e[i*4+3] ); }
	void reverse(){ for( int i=0; i<16; i++ ) e[i] = -e[i]; }
	void reverse( Matrix4X4 &a ){ for( int i=0; i<16; i++ ) e[i] = -a.e[i]; }
	void subtract( Matrix4X4 &a ){ for( int i=0; i<16; i++ ) e[i] -= a.e[i]; }
	void subtract( Matrix4X4 &a, Matrix4X4 &b ){ for( int i=0; i<16; i++ ) e[i] = a.e[i] - b.e[i]; }
	void exchange( double &a, double &b ){ double c = a; a = b; b = c; }
	void transpose(){ this->exchange( e[1], e[4] ); this->exchange( e[2], e[8] ); this->exchange( e[3], e[12] );
		this->exchange( e[6], e[9] ); this->exchange( e[7], e[13] ); this->exchange( e[11], e[14] ); }
	void transpose( Matrix4X4 &m ){ *this = m; this->transpose(); }
	void unit(){ for( int i=0; i<16; i++) e[i] = i%5==0 ? 1.0 : 0.0; }
	void zero(){ for( int i=0; i<16; i++) e[i] = 0.0; }
	Matrix4X4(){};
	Matrix4X4( double a1, double a2, double a3, double a4, double b1, double b2, double b3, double b4,
		double c1, double c2, double c3, double c4, double d1, double d2, double d3, double d4 )
		{ this->set( a1, a2, a3, a4, b1, b2, b3, b4, c1, c2, c3, c4, d1, d2, d3, d4 ); }
	Matrix4X4( double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3 )
		{ this->set( a1, a2, a3, b1, b2, b3, c1, c2, c3 ); }
	Matrix4X4( double *a ){ e[0]=a[0]; e[1]=a[1]; e[2]=a[2]; e[4]=a[3]; e[5]=a[4]; e[6]=a[5]; e[8]=a[6]; e[9]=a[7]; e[10]=a[8]; }
	Matrix4X4 operator +( Matrix4X4 &a ){ Matrix4X4 b; b.add( *this, a ); return b; }
	Matrix4X4 operator +=( Matrix4X4 &a ){ this->add( a ); return *this; }
	Matrix4X4 operator -( Matrix4X4 &a ){ Matrix4X4 b; b.subtract( *this, a ); return b; }
	Matrix4X4 operator -=( Matrix4X4 &a ){ this->subtract( a ); return *this; }
	Matrix4X4 operator -(){ Matrix4X4 a; a.reverse( *this ); return a; }
	Matrix4X4 operator *( double k ){ Matrix4X4 a; a.multiply( k, *this ); return a; }
	Matrix4X4 operator *( Matrix4X4 &a ){ Matrix4X4 b; b.multiply( *this, a ); return b; }
	Matrix4X4 operator *=( double k ){ this->multiply( k ); return *this; }
	Matrix4X4 operator /=( double k ){ this->multiply( 1.0/k ); return *this; }
	friend Matrix4X4 operator *( double k, Matrix4X4 &a ){ return a * k; }
	Vector operator *( Vector &a ){ Vector b; b.affine( *this, a ); return b; }
};

void Vector::affine( class Matrix4X4 &matrix, Vector &a ){
	x = a.x*matrix.e[0]+a.y*matrix.e[1]+a.z*matrix.e[2]+matrix.e[3];
	y = a.x*matrix.e[4]+a.y*matrix.e[5]+a.z*matrix.e[6]+matrix.e[7];
	z = a.x*matrix.e[8]+a.y*matrix.e[9]+a.z*matrix.e[10]+matrix.e[11];
}

void Vector::affine( Vector &a, class Matrix4X4 &matrix ){
	x = a.x*matrix.e[0]+a.y*matrix.e[4]+a.z*matrix.e[8]+matrix.e[12];
	y = a.x*matrix.e[1]+a.y*matrix.e[5]+a.z*matrix.e[9]+matrix.e[13];
	z = a.x*matrix.e[2]+a.y*matrix.e[6]+a.z*matrix.e[10]+matrix.e[14];
}

void Vector::linear( class Matrix4X4 &matrix, Vector &a ){
	x = a.x*matrix.e[0]+a.y*matrix.e[1]+a.z*matrix.e[2];
	y = a.x*matrix.e[4]+a.y*matrix.e[5]+a.z*matrix.e[6];
	z = a.x*matrix.e[8]+a.y*matrix.e[9]+a.z*matrix.e[10];
}

void Vector::linear( Vector &a, class Matrix4X4 &matrix ){
	x = a.x*matrix.e[0]+a.y*matrix.e[4]+a.z*matrix.e[8];
	y = a.x*matrix.e[1]+a.y*matrix.e[5]+a.z*matrix.e[9];
	z = a.x*matrix.e[2]+a.y*matrix.e[6]+a.z*matrix.e[10];
}

class Matrix3X3{
public:
	double e[9];

	void set( double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3 ){
		e[0]=a1; e[1]=a2; e[2]=a3; e[3]=b1; e[4]=b2; e[5]=b3; e[6]=c1; e[7]=c2; e[8]=c3; }
	void set( double a1, double a2, double b1, double b2 ){ e[0]=a1; e[1]=a2; e[3]=b1; e[4]=b2; }
	void setColumn( int col, double a, double b, double c ){ e[col]=a; e[col+3]=b; e[col+6]=c; }
	void setRow( int row, double a, double b, double c ){ e[row*3]=a; e[row*3+1]=b; e[row*3+2]=c; }
	int  inverse( Matrix3X3 &m ){ return inverseMatrix( e, m.e, 3 ); }
	int  inverse(){ Matrix3X3 m = *this; return inverseMatrix( e, m.e, 3 ); }
	void multiply( double k ){ for( int i=0; i<9; i++) e[i]*=k; }
	void multiply( double k, Matrix3X3 &a ){ for( int i=0; i<9; i++) e[i]=k*a.e[i]; }
	void multiply( Matrix3X3 &a, Matrix3X3 &b ){
		e[0] = a.e[0]*b.e[0] + a.e[1]*b.e[3] + a.e[2]*b.e[6];
		e[1] = a.e[0]*b.e[1] + a.e[1]*b.e[4] + a.e[2]*b.e[7];
		e[2] = a.e[0]*b.e[2] + a.e[1]*b.e[5] + a.e[2]*b.e[8];
		e[3] = a.e[3]*b.e[0] + a.e[4]*b.e[3] + a.e[5]*b.e[6];
		e[4] = a.e[3]*b.e[1] + a.e[4]*b.e[4] + a.e[5]*b.e[7];
		e[5] = a.e[3]*b.e[2] + a.e[4]*b.e[5] + a.e[5]*b.e[8];
		e[6] = a.e[6]*b.e[0] + a.e[7]*b.e[3] + a.e[8]*b.e[6];
		e[7] = a.e[6]*b.e[1] + a.e[7]*b.e[4] + a.e[8]*b.e[7];
		e[8] = a.e[6]*b.e[2] + a.e[7]*b.e[5] + a.e[8]*b.e[8];
	}
	void print(){ for( int i=0; i<3; i++ ) printf( "%f %f %f\n", e[i*3], e[i*3+1], e[i*3+2] ); }
	void exchange( double &a, double &b ){ double c = a; a = b; b = c; }
	void transpose(){ this->exchange( e[1], e[3] ); this->exchange( e[2], e[6] ); this->exchange( e[5], e[7] ); }
	void unit(){ for( int i=0; i<9; i++) e[i] = i%4==0 ? 1.0 : 0.0; }
	void zero(){ for( int i=0; i<9; i++) e[i] = 0.0; }
	Matrix3X3(){};
	Matrix3X3( double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, double c3 )
		{ this->set( a1, a2, a3, b1, b2, b3, c1, c2, c3 ); }
	Matrix3X3( double a1, double a2, double b1, double b2 ){ this->set( a1, a2, b1, b2 ); }
	Matrix3X3( double *a ){ for( int i=0; i<9; i++) e[i] = a[i]; }
	Matrix3X3 operator *( double k ){ Matrix3X3 a; a.multiply( k, *this ); return a; }
	Matrix3X3 operator *( Matrix3X3 a ){ Matrix3X3 b = *this; this->multiply( b, a ); return *this; }
	friend Matrix3X3 operator *( double k, Matrix3X3 &a ){ return a * k; }
	Vector   operator *( Vector &a ){ Vector b; b.linear( *this, a ); return b; }
	Vector2D operator *( Vector2D &a ){ Vector2D b; b.affine( *this, a ); return b; }
};

void Vector::linear( class Matrix3X3 &matrix, Vector &a ){
	x = a.x*matrix.e[0]+a.y*matrix.e[1]+a.z*matrix.e[2];
	y = a.x*matrix.e[3]+a.y*matrix.e[4]+a.z*matrix.e[5];
	z = a.x*matrix.e[6]+a.y*matrix.e[7]+a.z*matrix.e[8];
}

void Vector::linear( Vector &a, class Matrix3X3 &matrix ){
	x = a.x*matrix.e[0]+a.y*matrix.e[3]+a.z*matrix.e[6];
	y = a.x*matrix.e[1]+a.y*matrix.e[4]+a.z*matrix.e[7];
	z = a.x*matrix.e[2]+a.y*matrix.e[5]+a.z*matrix.e[8];
}

void Vector2D::affine( class Matrix3X3 &matrix, Vector2D &a ){
	x = a.x*matrix.e[0]+a.y*matrix.e[1]+matrix.e[2];
	y = a.x*matrix.e[3]+a.y*matrix.e[4]+matrix.e[5];
}

void Vector2D::affine( Vector2D &a, class Matrix3X3 &matrix ){
	x = a.x*matrix.e[0]+a.y*matrix.e[3]+matrix.e[6];
	y = a.x*matrix.e[1]+a.y*matrix.e[4]+matrix.e[7];
}

void Vector2D::linear( class Matrix3X3 &matrix, Vector2D &a ){
	x = a.x*matrix.e[0]+a.y*matrix.e[1];
	y = a.x*matrix.e[3]+a.y*matrix.e[4];
}

void Vector2D::linear( Vector2D &a, class Matrix3X3 &matrix ){
	x = a.x*matrix.e[0]+a.y*matrix.e[3];
	y = a.x*matrix.e[1]+a.y*matrix.e[4];
}

class Matrix2X2{
public:
	double e[4];

	void set( double a1, double a2, double b1, double b2 ){ e[0]=a1; e[1]=a2; e[2]=b1; e[3]=b2; }
	int  inverse( Matrix2X2 &m ){ return inverseMatrix( e, m.e, 2 ); }
	int  inverse(){ Matrix2X2 m = *this; return inverseMatrix( e, m.e, 2 ); }
	void multiply( double k ){ for( int i=0; i<4; i++) e[i]*=k; }
	void multiply( double k, Matrix2X2 &a ){ for( int i=0; i<4; i++) e[i]=k*a.e[i]; }
	void multiply( Matrix2X2 &a, Matrix2X2 &b ){
		e[0] = a.e[0]*b.e[0] + a.e[1]*b.e[2];
		e[1] = a.e[0]*b.e[1] + a.e[1]*b.e[3];
		e[2] = a.e[2]*b.e[0] + a.e[3]*b.e[2];
		e[3] = a.e[2]*b.e[1] + a.e[3]*b.e[3];
	}
	void print(){ for( int i=0; i<2; i++ ) printf( "%f %f\n", e[i*2], e[i*2+1] ); }
	void exchange( double &a, double &b ){ double c = a; a = b; b = c; }
	void transpose(){ this->exchange( e[1], e[2] ); }
	void unit(){ for( int i=0; i<4; i++) e[i] = i%3==0 ? 1.0 : 0.0; }
	void zero(){ for( int i=0; i<4; i++) e[i] = 0.0; }
	Matrix2X2(){};
	Matrix2X2( double a1, double a2, double b1, double b2 ){ this->set( a1, a2, b1, b2 ); }
	Matrix2X2( double *a ){ for( int i=0; i<4; i++) e[i] = a[i]; }
	Matrix2X2 operator *( double k ){ Matrix2X2 a; a.multiply( k, *this ); return a; }
	Matrix2X2 operator *( Matrix2X2 a ){ Matrix2X2 b = *this; this->multiply( b, a ); return *this; }
	friend Matrix2X2 operator *( double k, Matrix2X2 &a ){ return a * k; }
	Vector2D operator *( Vector2D &a ){ Vector2D b; b.linear( *this, a ); return b; }
};

void Vector2D::linear( class Matrix2X2 &matrix, Vector2D &a ){
	x = a.x*matrix.e[0]+a.y*matrix.e[1];
	y = a.x*matrix.e[2]+a.y*matrix.e[3];
}

void Vector2D::linear( Vector2D &a, class Matrix2X2 &matrix ){
	x = a.x*matrix.e[0]+a.y*matrix.e[2];
	y = a.x*matrix.e[1]+a.y*matrix.e[3];
}

#endif //_MATRIX_H
